@@ -12,7 +12,7 @@ NULL
 #'
 #' ## Authentication
 #'
-#' `chat_cortex()` picks up the following ambient Snowflake credentials:
+#' `chat_cortex_analyst()` picks up the following ambient Snowflake credentials:
 #'
 #' - A static OAuth token defined via the `SNOWFLAKE_TOKEN` environment
 #'   variable.
@@ -64,7 +64,7 @@ chat_cortex_analyst <- function(
   model_spec = NULL,
   model_file = NULL,
   api_args = list(),
-  echo = c("none", "text", "all")
+  echo = c("none", "output", "all")
 ) {
   check_string(account, allow_empty = FALSE)
   check_string(model_spec, allow_empty = FALSE, allow_null = TRUE)
@@ -79,7 +79,8 @@ chat_cortex_analyst <- function(
   check_function(credentials, allow_null = TRUE)
   credentials <- credentials %||% default_snowflake_credentials(account)
 
-  provider <- ProviderCortex(
+  provider <- ProviderSnowflakeCortexAnalyst(
+    name = "Snowflake/CortexAnalyst",
     account = account,
     credentials = credentials,
     model_spec = model_spec,
@@ -87,44 +88,20 @@ chat_cortex_analyst <- function(
     extra_args = api_args
   )
 
-  Chat$new(provider = provider, turns = NULL, echo = echo)
+  Chat$new(provider = provider, echo = echo)
 }
 
-#' Create a chatbot that speaks to the Snowflake Cortex Analyst
-#'
-#' @description
-#' `r lifecycle::badge("deprecated")`
-#'
-#' [chat_cortex()] was renamed to [chat_cortex_analyst()] to distinguish it from
-#' the more general-purpose Snowflake Cortex chat function, [chat_snowflake()].
-#'
-#' @inheritParams chat_cortex_analyst
-#' @keywords internal
-#' @export
-chat_cortex <- function(
-  account = snowflake_account(),
-  credentials = NULL,
-  model_spec = NULL,
-  model_file = NULL,
-  api_args = list(),
-  echo = c("none", "text", "all")
-) {
-  lifecycle::deprecate_warn("0.1.1", "chat_cortex()", "chat_cortex_analyst()")
-  chat_cortex_analyst(
-    account = account,
-    credentials = credentials,
-    model_spec = model_spec,
-    model_file = model_file,
-    api_args = api_args,
-    echo = echo
-  )
-}
-
-ProviderCortex <- new_class(
-  "ProviderCortex",
+ProviderSnowflakeCortexAnalyst <- new_class(
+  "ProviderSnowflakeCortexAnalyst",
   parent = Provider,
-  constructor = function(account, credentials, model_spec = NULL,
-                         model_file = NULL, extra_args = list()) {
+  constructor = function(
+    name,
+    account,
+    credentials,
+    model_spec = NULL,
+    model_file = NULL,
+    extra_args = list()
+  ) {
     base_url <- snowflake_url(account)
     extra_args <- compact(list2(
       semantic_model = model_spec,
@@ -132,7 +109,12 @@ ProviderCortex <- new_class(
       !!!extra_args
     ))
     new_object(
-      Provider(base_url = base_url, extra_args = extra_args),
+      Provider(
+        name = name,
+        base_url = base_url,
+        extra_args = extra_args,
+        model = ""
+      ),
       account = account,
       credentials = credentials
     )
@@ -144,13 +126,24 @@ ProviderCortex <- new_class(
   )
 )
 
+provider_cortex_test <- function(..., credentials = function(account) list()) {
+  ProviderSnowflakeCortexAnalyst(
+    name = "Cortex",
+    account = "testorg-test_account",
+    credentials = credentials,
+    ...
+  )
+}
+
 # See: https://docs.snowflake.com/en/developer-guide/snowflake-rest-api/reference/cortex-analyst
 #      https://docs.snowflake.com/en/user-guide/snowflake-cortex/cortex-analyst/tutorials/tutorial-1#step-3-create-a-streamlit-app-to-talk-to-your-data-through-cortex-analyst
-method(chat_request, ProviderCortex) <- function(provider,
-                                                 stream = TRUE,
-                                                 turns = list(),
-                                                 tools = list(),
-                                                 type = NULL) {
+method(chat_request, ProviderSnowflakeCortexAnalyst) <- function(
+  provider,
+  stream = TRUE,
+  turns = list(),
+  tools = list(),
+  type = NULL
+) {
   if (length(tools) != 0) {
     cli::cli_abort("Tools are not supported by Cortex.")
   }
@@ -160,10 +153,10 @@ method(chat_request, ProviderCortex) <- function(provider,
 
   req <- request(provider@base_url)
   req <- req_url_path_append(req, "/api/v2/cortex/analyst/message")
-  req <- req_headers(req, !!!provider@credentials(), .redact = "Authorization")
+  req <- ellmer_req_credentials(req, provider@credentials)
   req <- req_retry(req, max_tries = 2)
   req <- ellmer_req_timeout(req, stream)
-  req <- req_user_agent(req, snowflake_user_agent())
+  req <- ellmer_req_user_agent(req, Sys.getenv("SF_PARTNER"))
 
   # Snowflake doesn't document the error response format for Cortex Analyst at
   # this time, but empirically errors look like the following:
@@ -189,7 +182,10 @@ method(chat_request, ProviderCortex) <- function(provider,
 
 # Cortex -> ellmer --------------------------------------------------------------
 
-method(stream_parse, ProviderCortex) <- function(provider, event) {
+method(stream_parse, ProviderSnowflakeCortexAnalyst) <- function(
+  provider,
+  event
+) {
   # While undocumented, Cortex seems to mostly follow OpenAI API conventions for
   # streaming, although Cortex uses a specific JSON payload rather than in-band
   # signalling that the stream has ended.
@@ -200,7 +196,10 @@ method(stream_parse, ProviderCortex) <- function(provider, event) {
   }
 }
 
-method(stream_text, ProviderCortex) <- function(provider, event) {
+method(stream_text, ProviderSnowflakeCortexAnalyst) <- function(
+  provider,
+  event
+) {
   if (!is.null(event$status_message)) {
     # Skip status messages like "Interpreting question" or "Generating
     # suggestions".
@@ -222,12 +221,17 @@ method(stream_text, ProviderCortex) <- function(provider, event) {
     }
   } else {
     cli::cli_abort(
-      "Unknown chunk type {.str {event$type}}.", .internal = TRUE
+      "Unknown chunk type {.str {event$type}}.",
+      .internal = TRUE
     )
   }
 }
 
-method(stream_merge_chunks, ProviderCortex) <- function(provider, result, chunk) {
+method(stream_merge_chunks, ProviderSnowflakeCortexAnalyst) <- function(
+  provider,
+  result,
+  chunk
+) {
   if (!is.null(chunk$status)) {
     # Skip status messages.
     result
@@ -244,11 +248,13 @@ method(stream_merge_chunks, ProviderCortex) <- function(provider, result, chunk)
     # Only suggestion-type chunks are emitted piecemeal.
     if (identical(chunk$type, "suggestions")) {
       result[[idx]]$suggestions <- c(
-        result[[idx]]$suggestions, elt$suggestions
+        result[[idx]]$suggestions,
+        elt$suggestions
       )
     } else {
       cli::cli_abort(
-        "Unmergeable chunk type {.str {chunk$type}}.", .internal = TRUE
+        "Unmergeable chunk type {.str {chunk$type}}.",
+        .internal = TRUE
       )
     }
     result
@@ -267,25 +273,21 @@ cortex_chunk_to_message <- function(x) {
     )
   } else {
     cli::cli_abort(
-      "Unknown chunk type {.str {x$type}}.", .internal = TRUE
+      "Unknown chunk type {.str {x$type}}.",
+      .internal = TRUE
     )
   }
 }
 
-method(value_turn, ProviderCortex) <- function(provider, result, has_type = FALSE) {
-  if (!is_named(result)) { # streaming
-    role <- "assistant"
-    content <- result
-  } else {
-    role <- result$role
-    if (role == "analyst") {
-      role <- "assistant"
-    }
-    content <- result$content
-  }
+method(value_turn, ProviderSnowflakeCortexAnalyst) <- function(
+  provider,
+  result,
+  has_type = FALSE
+) {
+  is_streaming <- !is_named(result)
+  content <- if (is_streaming) result else result$content
 
-  Turn(
-    role = role,
+  assistant_turn(
     contents = lapply(content, function(x) {
       if (x$type == "text") {
         if (!has_name(x, "text")) {
@@ -294,7 +296,9 @@ method(value_turn, ProviderCortex) <- function(provider, result, has_type = FALS
         ContentText(x$text)
       } else if (identical(x$type, "suggestions")) {
         if (!has_name(x, "suggestions")) {
-          cli::cli_abort("'suggestions'-type content must have a 'suggestions' field.")
+          cli::cli_abort(
+            "'suggestions'-type content must have a 'suggestions' field."
+          )
         }
         ContentSuggestions(unlist(x$suggestions))
       } else if (identical(x$type, "sql")) {
@@ -303,7 +307,10 @@ method(value_turn, ProviderCortex) <- function(provider, result, has_type = FALS
         }
         ContentSql(x$statement)
       } else {
-        cli::cli_abort("Unknown content type {.str {x$type}} in response.", .internal = TRUE)
+        cli::cli_abort(
+          "Unknown content type {.str {x$type}} in response.",
+          .internal = TRUE
+        )
       }
     })
   )
@@ -315,7 +322,10 @@ method(value_turn, ProviderCortex) <- function(provider, result, has_type = FALS
 # Cortex supports not only "text" content, but also bespoke "suggestions" and
 # "sql" types.
 
-method(as_json, list(ProviderCortex, Turn)) <- function(provider, x) {
+method(as_json, list(ProviderSnowflakeCortexAnalyst, Turn)) <- function(
+  provider,
+  x
+) {
   role <- x@role
   if (role == "assistant") {
     role <- "analyst"
@@ -326,7 +336,10 @@ method(as_json, list(ProviderCortex, Turn)) <- function(provider, x) {
   )
 }
 
-method(as_json, list(ProviderCortex, ContentText)) <- function(provider, x) {
+method(as_json, list(ProviderSnowflakeCortexAnalyst, ContentText)) <- function(
+  provider,
+  x
+) {
   list(type = "text", text = x@text)
 }
 
@@ -336,7 +349,13 @@ ContentSuggestions <- new_class(
   properties = list(suggestions = class_character)
 )
 
-method(as_json, list(ProviderCortex, ContentSuggestions)) <- function(provider, x) {
+method(
+  as_json,
+  list(ProviderSnowflakeCortexAnalyst, ContentSuggestions)
+) <- function(
+  provider,
+  x
+) {
   list(type = "suggestions", suggestions = as.list(x@suggestions))
 }
 
@@ -367,7 +386,10 @@ ContentSql <- new_class(
   properties = list(statement = prop_string())
 )
 
-method(as_json, list(ProviderCortex, ContentSql)) <- function(provider, x) {
+method(as_json, list(ProviderSnowflakeCortexAnalyst, ContentSql)) <- function(
+  provider,
+  x
+) {
   list(type = "sql", statement = x@statement)
 }
 
