@@ -13,11 +13,6 @@ test_that("can make simple streaming request", {
   expect_match(paste0(unlist(resp), collapse = ""), "2")
 })
 
-# Common provider interface -----------------------------------------------
-
-test_that("respects turns interface", {
-  chat_fun <- chat_vllm_test
-
-  test_turns_system(chat_fun)
-  test_turns_existing(chat_fun)
+test_that("can list models", {
+  test_models(\(...) models_vllm("https://llm.nrp-nautilus.io/"))
 })
